#################################################################################
# Created       : Martin Foster
# Modified      : 07/08/2000
#################################################################################
#
# Crypto - Module hiding the complexity behind cryptographic uses in Ethereal 
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################
package Webchat::Crypto;					# Module name

use strict;							# Strict variable enforcement
use MIME::Base64;						# Base64 Encoding


#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# decryptParam
#
# Will take a reference to a hash, seperate it together using : as the seperator
# then calculate a MD5 hash compare and return 1 for good.

sub decryptParam
{
	#####################
	# Data members
	my $cgi		= shift;				# Common Gateway interface
	my $crypt	= shift;				# Crypted parameter list
	my $hash	= shift;				# Hash value

	my $string;						# String to work on


	#####################
	# Program area

	# Manipulate string
	$string = $cgi->unescape($crypt);
	$string = decode_base64($string);

	# Split and return
	%{$hash} = split(/\f/, $string);


	# Make sure it always exists
	$hash->{'MORI'} = (defined($hash->{'MORI'})) ? $hash->{'MORI'} : 0;

	# Session timeouts
	if ((time - $hash->{'MORI'}) < 3600)
	{
		# Cleanup
		delete $hash->{'MORI'};

		# Retrun true
		return 1;
	}

	# Return false if reached
	return 0;
}


#####################
# encryptParam
#
# Will take a flatened hash (key/pair), join it together using : as the seperator
# calculate a MD5 hash then encrypt the entire line using the provided key.

sub encryptParam
{
	#####################
	# Data members
	my $cgi 	= shift;				# Common gateway interface
	my $string;						# String to work on


	#####################
	# Program area

	# Concatenante to string
	$string = join("\f", @_);

	# Manipulate string
	$string = encode_base64($string, '');
	$string = $cgi->escape($string);

	# Return said string
	return $string;
}


#####################
# makeSalt
#
# Produces a salt using rand() and the Digest::MD5 hash.  Which once truncated to
# fourteen characters will be returned to parents function.

sub makeSalt
{
	#####################
	# Data members
	my $salt;						# Salt value

	#####################
	# Program area	
	
	# Generate hash, truncate then return
	$salt = md5_hex(rand());
	return substr($salt, 0, 14);
}

1;								# Return true
