#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 06/22/2001
#################################################################################
#
# Chat configuration - Script part of Webchat designed to configure MySQL for use 
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Webchat::Audit;						# Audit trail
use Webchat::Database;						# Database handler
use Webchat::Login;						# Login controls
use Webchat::Param;						# Parameter control

#################################################################################
# Data Members
#################################################################################

my $Database;							# Database handle
my $cgi;							# CGI Handle

my %param;							# Parameter list

#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# New CGI handle
	Webchat::Param::GetParam($cgi, \%param);		# Param to Hash
	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	if (Webchat::Login::GetVerificationAdmin($Database, $cgi, \%param))
	{
		print $cgi->p($cgi->h1('Webchat Tag Administration'));
		print $cgi->start_form();			# Start form

		DataCycler($Database, $cgi, \%param);

		print $cgi->p($cgi->submit(), $cgi->reset()), "\n<br>\n";
		Webchat::Param::EmbedNormal($Database, $cgi, 'USER', $param{'USER'}, 'CRYPT', $param{'CRYPT'});
		print $cgi->end_form(), "\n";
	}

	Webchat::Database::DocumentGetFooter($Database);        # HTML Footer
	$Database->disconnect();				# Disconnect database	


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Data Cycler
#
# A simple system designed to make the system more adaptable to new sections 
# being added to the system.   

sub DataCycler
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi      = shift;					# CGI filehandle
	my $param    = shift;					# Hash of parameters					
	my $choice   = shift;					# Choice of hash to open

	my @sections;						# List of sections
	my $section;						# Singular section


	#####################
	# Program area

	# Retreive sections
	Webchat::Database::GetList($Database, \@sections, 'SELECT DISTINCT(TagType) FROM Tag');


	# Cycle through sections
	foreach $section (sort @sections)
	{
		DataHandler($Database, $cgi, $param, $section);
	}
}


#####################
# Data Handler
#
# Takes in a CGI handle and the name of the hash to activate and displays 
# information within those hashes.   If changes, will make the change neccesary.

sub DataHandler							# CGI, Number for hash
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi      = shift;					# CGI filehandle
	my $param    = shift;					# Hash of parameters					
	my $dbm	     = shift;					# Hash being used

	my $choice;						# Current selection

	my $key;						# Arbitrary key
	my $value;						# Arbitrary value
	my $data;						# Data of hash

	my %hash;						# Arbitrary hash


	#####################
	# Program area


	# Link with hash
	Webchat::Database::GetHash($Database, \%hash, $dbm);


	# Display
	DisplayHandlerHeading($cgi, $dbm);
	DisplayHandlerTop($cgi, $dbm);
	DisplayHandlerTitle($cgi, $dbm);	

	# Loop through and display information
	foreach $choice (sort keys %hash)
	{
		# If diffrent change
		if ((defined($param->{"$dbm$choice"})) && ($param->{"$dbm$choice"} ne $hash{$choice})) 
		{
			$data = $param->{"$dbm$choice"};

			# Update values
			$hash{$choice} = $data;

			$Database->do("UPDATE Tag
				SET TagData=?
				WHERE TagName=?
				  AND TagType=?", {},

				 $data,
				 $choice,
				 $dbm);

			# Display fact of change
			DisplayHandlerNewInfo($cgi, $dbm, $choice, '*', $hash{$choice});

			# Audit Trail
			Webchat::Audit::TagChange($Database, $cgi, $param, $choice);
		}
			
		# Else display as no one really cares
		else
		{
			DisplayHandlerInfo($cgi, $dbm, $choice, '&nbsp;', $hash{$choice});
		}
	}

	DisplayHandlerBottom($cgi, $dbm);
}

#####################
# Display Handler
#
# Make formatted tables based on information received and options. 

sub DisplayHandler
{
	#####################
	# Data members
	my $cgi    = shift;					# CGI filehandle
	my $choice = shift;					# Choice of hash to open
	my $dbm    = shift;					# DBM being worked on

	my @title = qw(Key Value);				# Titles


	#####################
	# Program area

	# Top of Table
	if ($choice == 0)					# Table top
	{  
		print "<TABLE COLS=3 WIDTH=\"100%\">\n";
	}

	# Title section of table
	elsif ($choice == 1)					# Table title
	{
		print "\t<TR>\n";				# Row beginning

		# Print first three title columns
		print "\t\t<TD WIDTH=\"200\"><B><FONT SIZE=+1>$title[0]</FONT></B></TD>\n";
		print "\t\t<TD WIDTH=\"10\">&nbsp;</TD>\n";
		print "\t\t<TD><FONT SIZE=+1><B>$title[1]</FONT></B></TD>\n";		

		print "\t</TR>\n";				# Row end
	}

	# Information row of table
	elsif ($choice == 2)					# Table data
	{
		print "\t<TR>\n";				# Row beginning

		# Print three columns of data
		print "\t\t<TD><B>$_[0]</B></TD>\n";
		print "\t\t<TD><FONT COLOR=\"\#00CC00\">$_[1]</FONT></TD>\n";
		print "\t\t<TD>", $cgi->textfield("$dbm$_[0]",$_[2],80,1000), "</TD>\n";

		print "\t</TR>\n";				# Row end
	}

	# Information row of table
	elsif ($choice == 3)					# Table data
	{
		print "\t<TR>\n";				# Row beginning

		# Print three columns of data
		print "\t\t<TD><B>$_[0]</B></TD>\n";
		print "\t\t<TD><FONT COLOR=\"\#FF0000\">$_[1]</FONT></TD>\n";
		print "\t\t<TD>", $cgi->textfield("$dbm$_[0]",$_[2],80,1000), "</TD>\n";

		print "\t</TR>\n";				# Row end
	}

	# Bottom of table
	elsif ($choice == 4)					# Table bottom
	{
		print "</TABLE>\n\n<BR>";
	}

	# Heading of table
	elsif ($choice == 5)					# Table bottom
	{
		# Arrange title
		$dbm =~ tr/[a-z]/[A-Z]/;

		# Print title
		print "<BR>\n<BR>\n<B><FONT SIZE=+2>$dbm</FONT></B>\n<BR>\n";
	}
}

# Alias table top
sub DisplayHandlerTop
{ DisplayHandler(shift, 0, shift); }

# Alias table bottom
sub DisplayHandlerBottom
{ DisplayHandler(shift, 4, shift); }

# Alias table title
sub DisplayHandlerTitle
{ DisplayHandler(shift, 1, shift); }

# Alias table info
sub DisplayHandlerInfo
{ DisplayHandler(shift, 2, shift, @_); }

# Alias table new info
sub DisplayHandlerNewInfo
{ DisplayHandler(shift, 3, shift, @_); }

# Alias table heading
sub DisplayHandlerHeading
{ DisplayHandler(shift, 5, shift); }
