#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 06/16/2001
#################################################################################
#
# Configure Cookie - Script part of Webchat designed to control Cookies
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Webchat::Crypto;						# Crypto library
use Webchat::Database;						# Database handler
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my %param;							# Parameter hash
my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Database::GetHashConfiguration($Database, \%config);
	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	CookieHandler($cgi, \%config, \%param); 
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	# Authenthication
	if ( Webchat::Login::GetVerificationNormal($Database, $cgi, \%param) )
	{
		print "$config{'TagTitle'}\n<BR>\n";		# Print title
		CookieDisplay($Database, $cgi, \%config, \%param);
	}


	Webchat::Database::DocumentGetFooter($Database);	# HTML footer
	$Database->disconnect();				# Disconnect database handle

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Cookie Handler
#
# Handles the addition and removal of Cookies.  This is of course based on the
# creation of a header, which is why it must be placed beforehand.

sub CookieHandler						# CGI, Config, Param
{
	#####################
	# Data members
	my $cgi      = shift;					# CGI handle

	my $config   = shift;					# Configuration hash
	my $param    = shift;					# Parameter handle

	my $cookie;						# Cookie
	my $crypt;						# Crypted password
	my $mori;						# Kill switch

	my %system;						# System Hash


	#####################a
	# Program area	

	# System Hash
	Webchat::Database::GetHashSystem($Database, \%system);

	# Assign Values
	# Kill switch
	$mori = time + 6696000;

	# Encrypted password
	$crypt = Webchat::Crypto::encryptParam($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'MORI', $mori);
	$crypt = $cgi->unescape($crypt);


	# Cookie Addition
	if (defined($param->{$config->{'TxtSubmitAddition'}}))
	{
		# Generate cookies
		$cookie = $cgi->cookie(
			 -NAME		=> 'CRYPT',
			 -VALUE		=> $crypt,
			 -DOMAIN	=> $system{'SetDomain'},
			 -EXPIRES 	=> "+1M"
			);


		# Cookie Enable Header
		print $cgi->header(-cookie=>$cookie);

		# Display Message
		$param->{'MESSAGE'} = $config->{'MsgCookieAdd'};
	}

	# Cookie Removal
	elsif (defined($param->{$config->{'TxtSubmitDelete'}}))
	{

		# Generate cookies
		$cookie = $cgi->cookie(
			 -NAME		=> 'CRYPT',
			 -VALUE		=> $crypt,
			 -DOMAIN	=> $system{'SetDomain'},
			 -EXPIRES 	=> "-1y"
			);

		# Cookie Enable Header
		print $cgi->header(-cookie=>$cookie);

		# Display Message
		$param->{'MESSAGE'} = $config->{'MsgCookieRem'};
	}

	# Standard header
	else
	{
		# Standard header
		print $cgi->header();
	}

	
	# Cleanup
	delete $param->{$config->{'TxtSubmitAddition'}};
	delete $param->{$config->{'TxtSubmitDelete'}};
}


#####################
# Cookie Display
#
# Will handle the prompt for the addition and removal of cookies from Etherea 
# Realms.   This of course will be a rather straight forward display.

sub CookieDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle

	my $config   = shift;					# Configuration hash
	my $param    = shift;					# Parameter handle

	my $add;						# Add button
	my $rem;						# Remove button


	#####################
	# Program area	

	if (defined($param{'MESSAGE'}))
	{
		print "<B>$param{MESSAGE}</B>";
		delete $param{'MESSAGE'};

	}
	
	# Buttons
	$add = $cgi->submit($config->{'TxtSubmitAddition'});
	$rem = $cgi->submit($config->{'TxtSubmitDelete'});


	# Start form
	print $cgi->start_form();

	# Form display
	# Display information
	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeValid($cgi, "$config{TagCookieInfo}", "$add<BR><BR>$rem");
	Webchat::Table::MakeBottom($cgi);

	# Form Embed
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param});
	print $cgi->end_form();
}
