#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 02/26/2001
#################################################################################
#
# Home - Home page component for Ethereal-Realms 
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Date;						# Date processing
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table creation


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my $expire;							# Expired homepages
my $upload;							# Uploaded file

my %param;							# Parameter hash
my %home;							# Home page hash

#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle
	$upload = $cgi->upload('FUPLOAD');			# Retreive upload handle

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Database::GetHashHome($Database, \%home);	# Retreive hash

	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	# Should system be active
	if ($home{'SetActive'})
	{
		# Verify login and continue with system
		if ( Webchat::Login::GetVerificationNormal($Database, $cgi, \%param) )
		{
			# Expired or not?
			($expire) = Webchat::Database::DataGetPuppeteerExpire($Database, $param{'USER'});

			# Account still valid
			if ($expire > time)
			{
				# Make sure Directory is sane 
				if (DirHandler($Database, $cgi, \%param, \%home))
				{
					# Display title
					print "$home{TagTitle}\n";

					unless (defined($param{'LISTING'}))
					{
						# Handling of folder
						HomeHandler($Database, $cgi, \%param, \%home, $upload);
					}
					else
					{
						# Directory listing
						DirListing($Database, $cgi, \%param, \%home);
					}
				}
			}
			else
			{
				# Display disabled message
				Webchat::Table::MakeTop($cgi);
				Webchat::Table::MakeSingle($cgi, $home{'MsgExpire'});
				Webchat::Table::MakeBottom($cgi);
			}
		}
	}
	else
	{
		# Display disabled message
		Webchat::Table::MakeTop($cgi);
		Webchat::Table::MakeSingle($cgi, $home{'MsgDisabled'});
		Webchat::Table::MakeBottom($cgi);
	}

	Webchat::Database::DocumentGetFooter($Database);	# HTML footer
	$Database->disconnect();				# Disconnect database handle

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Directory Handler 
#
# This will make sure that certain components of Ethereals Home page system,
# primarily the home page directory is in order.   This means creation, prompt
# or whatever is needed to get it going. 

sub DirHandler  						# Database, CGI, Parameter list, Board hash
{
	#####################
	# Data members
	my $Database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $home     = shift;					# Home page hash handle 
	my $message  = shift;					# Error message

	my $dir;						# Directory

	my $header;						# Header file
	my $readme;						# Readme file


	#####################
	# Program area

	# Retreive DIR
	($dir) = Webchat::Database::DataGetPuppeteerHome($Database, $param->{'USER'});

 	# Determin status 
	# Directory defined 
	if (defined($dir))
	{
		# Ensure existence
		unless (-d "$home->{SetRoot}$dir")
		{
			# Create and verify
			unless (mkdir("$home->{SetRoot}$dir", 0775))
			{
				# Display error message
				Webchat::Table::MakeTop($cgi);
				Webchat::Table::MakeSingle($cgi, $home->{'MsgFail'});
				Webchat::Table::MakeBlank($cgi);
				Webchat::Table::MakeSingle($cgi, "<CENTER>$!</CENTER>");
				Webchat::Table::MakeBottom($cgi);

				# Reset Database
				$Database->do("UPDATE Puppeteer SET PuppeteerHome=NULL WHERE PuppeteerLogin=?", {}, $param->{'USER'});	
				
				# Return False
				return 0;
			} 
		}

		# Retreive header and readme
		($header) = Webchat::Database::DocumentGetHomeHeader($Database);
		($readme) = Webchat::Database::DocumentGetHomeReadme($Database);

		# Write contents
		FileWrite("$home->{SetRoot}$dir/$home->{SetHEADER}", $header);
		FileWrite("$home->{SetRoot}$dir/$home->{SetREADME}", $readme);

		# Return true
		return 1;
	}

	# Directory creation
	elsif ((defined($param->{'DCREATE'})) && ($param->{'DCREATE'} !~ /\W+/g))
	{
		# Parameter handling
		$dir = $param->{'DCREATE'};
		delete $param->{'DCREATE'};


		# Ensure existence
		unless (-d "$home->{SetRoot}$dir")
		{
			# Create and verify
			# Successful
			if (mkdir("$home->{SetRoot}$dir", 0755))
			{
				# Retreive header and readme
				($header) = Webchat::Database::DocumentGetHomeHeader($Database);
				($readme) = Webchat::Database::DocumentGetHomeReadme($Database);

				# Write contents
				FileWrite("$home->{SetRoot}$dir/$home->{SetHEADER}", $header);
				FileWrite("$home->{SetRoot}$dir/$home->{SetREADME}", $readme);

				# Update Database
				$Database->do("UPDATE Puppeteer SET PuppeteerHome=? WHERE PuppeteerLogin=?", {}, 
					$dir,
					$param->{'USER'});
				
				# Return True
				return 1;
			} 

			# Fail
			else
			{
				# Display error message
				Webchat::Table::MakeTop($cgi);
				Webchat::Table::MakeSingle($cgi, $home->{'MsgFail'});
				Webchat::Table::MakeBlank($cgi);
				Webchat::Table::MakeSingle($cgi, "<CENTER>$!</CENTER>");
				Webchat::Table::MakeBottom($cgi);
			}
		}
		else
		{ DirHandler($Database, $cgi, $param, $home, $home->{'MsgExists'}); }
		
		# Return false
		return 0; 
	} 

	# New page setup
 	else	
	{
		# Start form
		print $cgi->start_form();

		# Display creation form
		Webchat::Table::MakeTop($cgi);

		# Message display
		if (defined($message)) { Webchat::Table::MakeSingle($cgi, $message); Webchat::Table::MakeBlank($cgi); }

		Webchat::Table::MakeSingle($cgi, $home->{'TagHome'});
		Webchat::Table::MakeSingle($cgi, $cgi->textfield('DCREATE', '', 55, 55));
		Webchat::Table::MakeBlank($cgi);
		Webchat::Table::MakeValid($cgi, $cgi->submit($home->{'TxtCreate'}), $cgi->reset($home->{'TxtReset'}));
		Webchat::Table::MakeBottom($cgi);

		# Embed and close form
		Webchat::Param::EmbedNormal($Database, $cgi, %{$param});
		print $cgi->end_form(), "\n";

		# Return false
		return 0;
	}
}

#####################
# Directory Listing
#
# Used to show a user all files in the folder and allow them to select it
# if needed.  This is used if directory indexing has been disabled.

sub DirListing							# Database, CGI, Param and Home
{
	#####################
	# Data members
	my $Database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $home     = shift;					# Home page hash handle 

	my $bytes;						# Individual size
	my $count;						# File count
	my $dir;						# Users Directory
	my $file;						# Individual file
	my $size;						# Size of files

	my $inline;						# Inline parameters
	my $url      = $cgi->url();				# Back to main

	my @files;						# File listing
	my @listing;						# Listing of files


	#####################
	# Program Area

	# Cleanup
	delete $param->{'LISTING'};


	# Data retreival
	# Retreive DIR
	($dir) = Webchat::Database::DataGetPuppeteerHome($Database, $param->{'USER'});

	# Retreive file listing
	($size, $count) = FileListing($home, "$home->{SetRoot}$dir", \@files); 

	# Generate inline
	($inline) = Webchat::Param::EmbedInline($Database, $cgi, %{$param});



	# Top of table
	Webchat::Table::MakeTop($cgi);

	# Back Button
	Webchat::Table::MakeSingle($cgi, "<CENTER><A HREF=\"$url$inline\">$home->{TagBack}</A></CENTER>");
	Webchat::Table::MakeBlank($cgi);

	# Url listing
	Webchat::Table::MakeSingle($cgi, $home->{'TagUrl'});
	Webchat::Table::MakeSingle($cgi, "<A HREF=\"$home->{SetUrl}$dir\" TARGET=\"_new\">$home->{SetUrl}$dir</A>");
	Webchat::Table::MakeBlank($cgi);

	# Directory listing
	Webchat::Table::MakeSingle($cgi, $home->{'TagListing'});

	# Cycle and list files
	foreach $file (sort(@files))
	{
		# Retreive file size
		$bytes = -s "$home->{SetRoot}$dir/$file";

		# Filename is escaped
		$file = $cgi->escape($file);

		# Display Filename
		Webchat::Table::MakeSingle($cgi, "<A HREF=\"$home->{SetUrl}$dir/$file\" TARGET=\"_new\">$home->{SetUrl}$dir/$file</A> ($bytes Bytes)");
	}

	# Useage Summary
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeSingle($cgi, $home->{'TagSummary'});
	Webchat::Table::MakeSingle($cgi, "$size $home->{TagStatusSize} $count $home->{TagStatusCount}");
	Webchat::Table::MakeBottom($cgi);

	# Bottom of table
	Webchat::Table::MakeBottom($cgi);
}

#####################
# File listing 
#
# Handles the recovery of file information from within a users personal home 
# folder.   This will return size and file count for convenience.

sub FileListing 						# Hash, Directory name, Array Ref 
{
	#####################
	# Data members
	my $home     = shift;					# Home hash
	my $dir      = shift;					# Directory name 
	my $files    = shift;					# Array of files

	my $file;						# File name
 
	my $count = 0;						# File count
	my $size  = 0;						# File size


	#####################
	# Program area

	# Open Directory Handle 
 	opendir(DIR, $dir) or return $size, $count;	

	# Read through directory listing
	while ($file = readdir(DIR))
	{
		# Possible skips
		next if ($file =~ /^\./);
		next if ($file eq $home->{'SetHEADER'});
		next if ($file eq $home->{'SetREADME'});			

		# Add to size and increment
		$size += -s "$dir/$file";
		$count++;

		# Append to array
		push(@{$files}, $file); 
	}	

	# Close handle
	closedir(DIR);
	
	# Return values
	return $size, $count;
}


#####################
# File Write 
#
# As the name implies, this subroutine is designed to hide some of the more
# complex features of file writing.  This system will write whatever was
# received in the parameter list.   Overwriting existing data. 

sub FileWrite 	 					 	# File name, contents	
{
	#####################
	# Data members
	my $file     = shift;					# Name of file
	my $contents = shift;					# Contents of file

	#####################
	# Program area

	# Open file
	if (open(FILE, ">$file"))
	{
		# Write out contents
		print FILE $contents;

		# Close handle
		close(FILE);
	}

	# Return error
	else
	{ return $!; }
}


#####################
# HomeHandler 
#
# Provides the interface to see all files within their home folders as well
# as allow for additions and removals as one sees fit.  

sub HomeHandler  						# Database, CGI, Parameter list, Board hash
{
	#####################
	# Data members
	my $Database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $home     = shift;					# Home page hash handle 
	my $upload   = shift;					# File upload

	my $count;						# File listing
	my $dir;						# Directory
	my $size;						# File size
	my $sorted;						# Sorted file CGI

	my $contents;						# File contents
	my $file;						# File name
	my $type;						# File type

	my $message;						# Error message

	my $inline;						# Inline parameters
	my $url      = $cgi->url();				# Back to main

	my @files;						# Files list
	my @sorted;						# Sorted files

	#####################
	# Program area

	# Retreive directory handle 
	($dir) = Webchat::Database::DataGetPuppeteerHome($Database, $param->{'USER'});

	# Operations
	# Upload field handle
	if (defined($upload))
	{
		# Retreive MIME Type
		$type = $cgi->uploadInfo($param->{'FUPLOAD'})->{'Content-Type'};

		# Filename handling
		$file = $param->{'FUPLOAD'}; 
		$file =~ m!([^/:\\]*)$!;
		$file = $1;  

		# Only allow specific MIME types
		if (($type =~ /^image/i) || ($type =~ /^text/i))
		{
			unless (($file =~ /^\./) || ($file eq $home->{'SetHEADER'}) || ($file eq $home->{'SetREADME'}))
			{
				# Retreive information
				while (<$upload>) { $contents = "$contents$_"; }
			
				# Write information
				FileWrite("$home->{SetRoot}$dir/$file", $contents); 

				# Message
				$message = $home->{'MsgCreate'};
			}
			# Otherwise error message
			else
			{ $message = $home->{'MsgFail'}; }
		}
		# Otherwise error message
		else
		{ $message = $home->{'MsgFail'}; }

		# Cleanup
		delete $param->{'FUPLOAD'};
		delete $param->{$home->{'TxtCreate'}};
	}

	# Remove files
	if (defined($param->{$home->{'TxtDelete'}}))
	{
		# Remove file
		unlink("$home->{SetRoot}$dir/$param->{'FSELECT'}");

		# Assign message
		$message = $home->{'MsgDelete'};

		# Cleanup
		delete $param->{$home->{'TxtDelete'}};
		delete $param->{'FSELECT'};
	}

	# Data handling
	# Retreive file listing
	($size, $count) = FileListing($home, "$home->{SetRoot}$dir", \@files); 

	# Sort and prepare 
	@sorted = sort(@files);
	$sorted = $cgi->scrolling_list('FSELECT',\@sorted,$sorted[0],15);

	# Generate inline
	($inline) = Webchat::Param::EmbedInline($Database, $cgi, %{$param}, 'LISTING', 'True');


	# HTML Display
	# Start form
	print $cgi->start_multipart_form();

	# Table display
	Webchat::Table::MakeTop($cgi);

	# Message
	if (defined($message)) { Webchat::Table::MakeSingle($cgi, $message); Webchat::Table::MakeBlank($cgi); }

	# File listing
	Webchat::Table::MakeSingle($cgi, $home->{'TagListing'});
	Webchat::Table::MakeValid($cgi, $sorted, $cgi->submit($home->{'TxtDelete'}));

	# Addition of files
	if ($size < ($home->{'SetSize'} * 1048576))
	{
		# Display addition area
		Webchat::Table::MakeBlank($cgi);
		Webchat::Table::MakeSingle($cgi, $home->{'TagAdd'});
		Webchat::Table::MakeSingle($cgi, $cgi->filefield('FUPLOAD', '', 40, 80));
		Webchat::Table::MakeSingle($cgi, $cgi->submit($home->{'TxtCreate'}));
	}
	# Display lack of capability
	else { Webchat::Table::MakeSingle($cgi, $home->{'MsgSpace'}); }

	# Url listing
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeSingle($cgi, $home->{'TagUrl'});
	Webchat::Table::MakeSingle($cgi, "<A HREF=\"$home->{SetUrl}$dir\" TARGET=\"blank\">$home->{SetUrl}$dir</A>");
	
	# Advanced listing
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeSingle($cgi, "<A HREF=\"$url$inline\">$home->{TagListingAdv}</A>");


	# Useage Summary
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeSingle($cgi, $home->{'TagSummary'});
	Webchat::Table::MakeSingle($cgi, "$size $home->{TagStatusSize} $count $home->{TagStatusCount}");
	Webchat::Table::MakeBottom($cgi);

	# Embed and close form
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param});
	print $cgi->end_form(), "\n";
}
